@@ -205,7 +205,7 @@ lock_self:
 #ifdef HA_CAS_IS_8B
 		 unlikely(!_HA_ATOMIC_CAS(((uint64_t *)&_GET_NEXT(fd, off)), (uint64_t *)&cur_list.u64, next_list.u64))
 #else
-		 unlikely(!_HA_ATOMIC_DWCAS(((long *)&_GET_NEXT(fd, off)), (uint32_t *)&cur_list.u32, &next_list.u32))
+		 unlikely(!_HA_ATOMIC_DWCAS(((long *)&_GET_NEXT(fd, off)), (uint32_t *)&cur_list.u32, (const uint32_t *)&next_list.u32))
 #endif
 	    );
 	next = cur_list.ent.next;
@@ -462,7 +462,6 @@ __decl_spinlock(log_lock);
 ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t npfx, const struct ist msg[], size_t nmsg, int nl)
 {
 	struct iovec iovec[32];
-	size_t totlen = 0;
 	size_t sent = 0;
 	int vec = 0;
 	int attempts = 0;
@@ -489,7 +488,6 @@ ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t
 		iovec[vec].iov_base = pfx->ptr;
 		iovec[vec].iov_len  = MIN(maxlen, pfx->len);
 		maxlen -= iovec[vec].iov_len;
-		totlen += iovec[vec].iov_len;
 		if (iovec[vec].iov_len)
 			vec++;
 		pfx++; npfx--;
