@@ -171,12 +171,13 @@ int hlua_fcn_pushfield(lua_State *L, struct field *field)
 const char *hlua_pushstrippedstring(lua_State *L, const char *str)
 {
 	const char *p;
-	const char *e;
+	int l;
 
 	for (p = str; HTTP_IS_LWS(*p); p++);
-	for (e = p + strlen(p) - 1; e > p && HTTP_IS_LWS(*e); e--);
 
-	return lua_pushlstring(L, p, e - p);
+	for (l = strlen(p); l && HTTP_IS_LWS(p[l-1]); l--);
+
+	return lua_pushlstring(L, p, l);
 }
 
 /* The three following functions are useful for adding entries
@@ -1301,6 +1302,7 @@ int hlua_proxy_pause(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - pause_proxy takes it */
 	pause_proxy(px);
 	return 0;
 }
@@ -1310,6 +1312,7 @@ int hlua_proxy_resume(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - resume_proxy takes it */
 	resume_proxy(px);
 	return 0;
 }
@@ -1319,6 +1322,7 @@ int hlua_proxy_stop(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - stop_proxy takes it */
 	stop_proxy(px);
 	return 0;
 }
